package basics.datagraph;

import java.io.File;
import java.util.Iterator;

import javatools.administrative.D;
import javatools.administrative.Parameters;
import javatools.filehandlers.TSVWriter;

public class DataGraphManager {

  /**
   * initialises the  yago classes and relations given a path 
   * @param path
   * @return
   */
  public static boolean init(String path) {
    if (!new File(path).exists()) {
      D.pl("Path does not exist " + path);
      return false;
    }
    File classes = new File(path, "YagoClasses");
    boolean ok = YagoClasses.loadFrom(classes);
    if (!ok) {
      return false;
    }
    File relations = null;
    relations = new File(path, "Relations");
    return Relations.loadFrom(relations);
  }

  /**
   * initialises the yago relations and classes, should the path not be
   * specified then the classes are loaded from the jar file.
   */
  public static void init() {
    File classes = null;
    if (Parameters.isDefined("yagoDefinitionsFolder")) {
      classes = new File(Parameters.getFile("yagoDefinitionsFolder"), "YagoClasses");
      YagoClasses.loadFrom(classes);
    } else {
      //      Announce.
      YagoClasses.loadFromClass();
    }
    File relations = null;
    if (Parameters.isDefined("yagoDefinitionsFolder")) {
      relations = new File(Parameters.getFile("yagoDefinitionsFolder"), "Relations");
      Relations.loadFrom(relations);
    } else {
      Relations.loadFromClass();
    }
  }

  /**
   * for testing
   * @param args
   * @throws Exception
   */
  public static void main2(String[] args) throws Exception {
    try {
      Parameters.init(args == null || args.length == 0 ? "yago.ini" : args[0]);
      DataGraphManager.init();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * creates the tsv files for relations and yagoclasses
   * @param args
   * @throws Exception
   */
  public static void main(String[] args) throws Exception {
    try {
      Parameters.init(args == null || args.length == 0 ? "yago.ini" : args[0]);
      DataGraphManager.init();
      DataGraphManager.writeAsTSV();
    } catch (Exception e) {
      e.printStackTrace();
    }
    System.out.println("Done");
  }

  public static void writeAsTSV() {
    // Relations
    Iterator<Relation> relations = Relations.values().iterator();
    try {
      TSVWriter writer = new TSVWriter("relations.tsv");
      TSVWriter writerVirtual = new TSVWriter("virtualRelations.tsv");
      while (relations.hasNext()) {
        TSVWriter use = null;
        Relation relation = relations.next();
        if (!relation.getClass().getName().equals("basics.datagraph.Relation")) {
          use = writerVirtual;
        } else {
          use = writer;
        }
        if (relation.name == null) {
          System.out.println("ERROR name for realtion is zero");
          continue;
        }
        if (relation.domain == null) {
          System.out.println("Domain is null");
        } else {
          use.write(relation.name, Relations.DOMAIN.name, relation.domain.name);
        }
        if (relation.range == null) {
          System.out.println("Range is null");
        } else {
          use.write(relation.name, Relations.RANGE.name, relation.range.name);
        }
        if (relation.superRelation != null) {
          use.write(relation.name, Relations.SUBCLASSOF.name, relation.superRelation.name);
        }
        Iterator<YagoClass> types = relation.types.iterator();
        while (types.hasNext()) {
          YagoClass type = types.next();
          use.write(relation.name, Relations.TYPE.name, type.name);
        }
        if (!relation.getClass().getName().equals("basics.datagraph.Relation")) {
          use.write(relation.name, "_hasVirtualRelation", relation.getClass().getName());
        }
      }
      writer.close();
      writerVirtual.close();
    } catch (Exception e) {
      e.printStackTrace();
    }
    try {
      // YagoClasses
      TSVWriter writerLiteral = new TSVWriter("literalYagoClasses.tsv");
      TSVWriter writerOther = new TSVWriter("otherYagoClasses.tsv");
      TSVWriter writerWordNet = new TSVWriter("wordNetClasses.tsv");
      Iterator<YagoClass> yagoclasses = YagoClasses.values().iterator();
      while (yagoclasses.hasNext()) {
        YagoClass yagoClass = yagoclasses.next();
        if (yagoClass.getClass().getName().equals("basics.datagraph.LiteralYagoClass")) {
          LiteralYagoClass literal = (LiteralYagoClass) yagoClass;
          writerLiteral.write(literal.name, "_isYagoClass", "LiteralYagoClass");
          if (literal.typeCheckPattern != null) {
            writerLiteral.write(literal.name, "_hasTypeCheckPattern", "\""+literal.typeCheckPattern.pattern()+"\"");
          }
          Iterator<YagoClass> superclasses = literal.superclasses.iterator();
          while (superclasses.hasNext()) {
            YagoClass superclass = superclasses.next();
            writerLiteral.write(literal.name, Relations.SUBCLASSOF.name, superclass.name);
          }
        } else if (yagoClass.getClass().getName().equals("basics.datagraph.WordNetClass")) {
          WordNetClass wordNet = (WordNetClass) yagoClass;
          writerWordNet.write(wordNet.name, "_isYagoClass", "WordNetClass");
          writerWordNet.write(wordNet.name, "_hasWordNetId", wordNet.wordNetId + "");
        } else {
          writerOther.write(yagoClass.name, "_isYagoClass", "YagoClass");
          Iterator<YagoClass> superclasses = yagoClass.superclasses.iterator();
          while (superclasses.hasNext()) {
            YagoClass superclass = superclasses.next();
            writerOther.write(yagoClass.name, Relations.SUBCLASSOF.name, superclass.name);
          }
        }
      }
      writerLiteral.close();
      writerOther.close();
      writerWordNet.close();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}
