package basics.datagraph;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Pattern;

/**
 * This class is part of the YAGO extractors (http://mpii.de/yago). It is licensed under the 
 * Creative Commons Attribution-Noncommercial-Share Alike 3.0 Unported License,
 * (http://creativecommons.org/licenses/by-nc-sa/3.0/) 
 * by the YAGO team (http://mpii.de/yago). 
 * 
 * This class represents the literal YAGO classes.<P>
 *
 * @author Fabian M. Suchanek*/
public class LiteralYagoClass extends YagoClass {

  /** This pattern tells if an entity can be an instance of this YagoClass*/
  protected Pattern typeCheckPattern;

  public LiteralYagoClass(String name, List<YagoClass> superclasses) {
    this(name, superclasses,(Pattern)null);
  }

  public LiteralYagoClass(String name, YagoClass superclass) {
    this(name,Arrays.asList(superclass));
  }
  
  public LiteralYagoClass(String name, List<YagoClass> superclasses, Pattern typeCheckPattern) {
    super(name, superclasses);
    this.typeCheckPattern = typeCheckPattern;
  }

  public LiteralYagoClass(String name, YagoClass superclass, Pattern typeCheckPattern) {
    this(name,Arrays.asList(superclass),typeCheckPattern);
  }

  public LiteralYagoClass(String name, YagoClass superclass, String typeCheckPattern) {
    this(name,Arrays.asList(superclass),typeCheckPattern);
  }

  public LiteralYagoClass(String name, List<YagoClass> superclasses, String typeCheckPattern) {
    this(name, superclasses, typeCheckPattern == null ? null : Pattern.compile(typeCheckPattern));
  }

  public LiteralYagoClass(String name, String typeCheckPattern) {
    this(name,new ArrayList<YagoClass>(),typeCheckPattern);
  }

  /** 1 if the class contains the instance, 0 for don't know and -1 for no*/
  public int hasInstance(String s) {
    if (typeCheckPattern == null) return (0);
    return (typeCheckPattern.matcher(s).matches() ? 1 : -1);
  }

  //  public static final LiteralYagoClass LITERAL = new LiteralYagoClass("yagoLiteral", ENTITY);
  //  public static final LiteralYagoClass BOOLEAN = new LiteralYagoClass("yagoBoolean", LITERAL, "true|false");
  //  public static final LiteralYagoClass NUMBER = new LiteralYagoClass("yagoNumber", LITERAL, NumberParser.FLOAT);
  //  public static final LiteralYagoClass RATIONAL = new LiteralYagoClass("yagoRationalNumber", NUMBER, NumberParser.FLOAT);
  //  public static final LiteralYagoClass INTEGER = new LiteralYagoClass("yagoInteger", RATIONAL, NumberParser.INT);
  //  public static final LiteralYagoClass NONNEGATIVEINTEGER = new LiteralYagoClass("yagoNonNegativeInteger", INTEGER, NumberParser.POSINT);
  //  public static final LiteralYagoClass STRING = new LiteralYagoClass("yagoString", LITERAL);
  //  public static final LiteralYagoClass WORD = new LiteralYagoClass("yagoWord", STRING);
  //  public static final LiteralYagoClass URL = new LiteralYagoClass("yagoURL", STRING, "[a-z]+://.+");
  //  public static final LiteralYagoClass TLD = new LiteralYagoClass("yagoTLD", STRING, "\\.([a-z]{2,5})");
  //  public static final LiteralYagoClass CHAR = new LiteralYagoClass("yagoChar", STRING, ".");
  //  public static final LiteralYagoClass TIMEINTERVAL = new LiteralYagoClass("yagoTimeInterval", LITERAL);
  //  public static final LiteralYagoClass TIMEPOINT = new LiteralYagoClass("yagoTimePoint", TIMEINTERVAL);
  //  public static final LiteralYagoClass DATE = new LiteralYagoClass("yagoDate", TIMEINTERVAL, DateParser.DATEPATTERN);
  //  public static final LiteralYagoClass YEAR = new LiteralYagoClass("yagoYear", TIMEINTERVAL, "[0-9]{1,4}");
  //  public static final LiteralYagoClass QUANTITY = new LiteralYagoClass("yagoQuantity", LITERAL);
  //  public static final LiteralYagoClass LENGTH = new LiteralYagoClass("yagoLength", QUANTITY, NumberParser.FLOAT + "#m");
  //  public static final LiteralYagoClass AREA = new LiteralYagoClass("yagoArea", QUANTITY, NumberParser.FLOAT + "#m\\^2");
  //  public static final LiteralYagoClass WEIGHT = new LiteralYagoClass("yagoWeight", QUANTITY, NumberParser.FLOAT + "#g");
  //  public static final LiteralYagoClass DENSITYPERAREA = new LiteralYagoClass("yagoDensityPerArea", QUANTITY, NumberParser.FLOAT + "#/km\\^2");
  //  public static final LiteralYagoClass DURATION = new LiteralYagoClass("yagoDuration", QUANTITY, NumberParser.FLOAT + "#s");
  //  public static final LiteralYagoClass MONETARYVALUE = new LiteralYagoClass("yagoMonetaryValue", QUANTITY, NumberParser.FLOAT + "#(dollar|euro)");
  //  public static final LiteralYagoClass IDENTIFIER = new LiteralYagoClass("yagoIdentifier", STRING, ".*");
  //  public static final LiteralYagoClass CALLINGCODE = new LiteralYagoClass("yagoCallingCode", NONNEGATIVEINTEGER, "[0-9]{1,8}");
  //  public static final LiteralYagoClass ISBN = new LiteralYagoClass("yagoISBN", IDENTIFIER, "((?:\\d+\\-){3}\\d+)");
}
