package basics.datagraph;

import java.util.Collection;

/**
 * This class is part of the YAGO extractors (http://mpii.de/yago). It is licensed under the 
 * Creative Commons Attribution-Noncommercial-Share Alike 3.0 Unported License,
 * (http://creativecommons.org/licenses/by-nc-sa/3.0/) 
 * by the YAGO team (http://mpii.de/yago). 
 * 
 * This class represents relations that are not stored in the database, but 
 * computed on the fly. Examples are smallerThan, greaterThan etc.
 * 
 * These relations have a method "holds", which computes whether the relation holds
 * between two entities (given as Strings).
 * 
 * @author Fabian M. Suchanek
 */
public abstract class VirtualRelation extends Relation {
    
  public VirtualRelation(String name, YagoClass domain, YagoClass range, Collection<YagoClass> types) {
    this(name,domain,range,null,types);
  }
  public VirtualRelation(String name, YagoClass domain, YagoClass range, Relation superRelation, Collection<YagoClass> types) {
    super(name, domain, range, superRelation, types);  
  }

  public VirtualRelation(String name, YagoClass domain, YagoClass range) {
    super(name,domain,range);
  }
  
  /** Tells whether this relation holds between two Strings */
  public abstract boolean holds(String x, String y);
  
}
