package converters;

import java.io.File;

import javatools.administrative.Announce;
import javatools.administrative.D;
import javatools.administrative.Parameters;

/** 
 * This class is part of the YAGO converters (http://mpii.de/yago). 
 * It is licensed under the Creative Commons Attribution License 
 * (http://creativecommons.org/licenses/by/3.0) 
 * by the YAGO team (http://mpii.de/yago).
 * 
 * This class calls the appropriate class method for conversions to
 * XML, RDFS, N3 and SQL 
 * 
 * @author Fabian M. Suchanek
 */
public class Convert {

  /** Calls the appropriate class method for conversions to
   * XML, RDFS and SQL */
  public static void main2(String[] args) throws Exception {
    String[] parameters = args != null && args.length == 2 ? new String[] { args[1] } : null;
    String dest = args != null && args.length > 0 && args.length < 3 ? args[0].toLowerCase() : null;
    if (D.equal(dest, "xml")) XMLConverter.main(parameters);
    else if (D.equal(dest, "rdf") || D.equal(dest, "rdfs")) RDFSConverter.main(parameters);
    else if (D.equal(dest, "db")) SQLConverter.main(parameters);
    else if (D.equal(dest, "n3") || D.equal(dest, "notation3")) N3Converter.main(parameters);
    else if (D.equal(dest, "dbpediaexport")) DBpediaExport.main(parameters);
    else if (D.equal(dest, "dbpedialink")) DBpediaLink.main(parameters);
    else Announce.help("     Converter for the YAGO ontology", "", "java converter.Convert XML|RDFS|DB|DBpediaExport|DBpediaLink|n3 [<ini-file>]", "", "Converts YAGO to XML, N3 or RDFS or reads it into a database.",
        "Requires detailed settings in <ini-file>. See the Readme.txt file for more information.", "If <ini-file> is not given, the program defaults to yago.ini.");
  }

  private static String[] options = { "DB", "XML", "RDFS", "n3", "DBpediaLink", "DBpediaExport", "alch" };

  /**
   * Start the porgram, one can eiter provide a ini file or the choice.
   * or all required parameters are requested. 
   */
  public static void main(String[] args) throws Exception {
    String initFile = args == null || args.length == 0 ? "yago.ini" : args[0];
    File f = new File(initFile);
    int choice = 0;
    if (f.exists()) {
      Parameters.init(f);
      String method = Parameters.get("convert","none");
      for (int i = 0; i < options.length; i++) {
        if (options[i].equalsIgnoreCase(method)) {
          choice = i + 1;
        }
      }
    }
    while (choice <= 0 || choice >= 8) {
      D.println("Welcome to the Yago ontology convertor");
      D.println("Please Choose from the following options:");
      D.println("-----------------------------------------------------------------");
      D.println("1. Transfer yago into a database (Oracle,MySQL,Postgres)");
      D.println("2. Convert YAGO to XML");
      D.println("3. Convert YAGO to RDFS");
      D.println("4. Convert YAGO to N3 (Notation 3)");
      D.println("5. Link YAGO and DBpedia by owl:sameAs statements");
      D.println("6. Export YAGO's class hierarchy for importing it into DBpedia");
      D.println("7. Converts YAGO to the DB format required by Alchemy");
      D.println("8. quit or use (ctrl+c)");
      D.pl("input:> ");
      try {
        choice = Integer.parseInt(D.r().trim());
      } catch (Exception e) {
        choice = 0;
      }
      if (choice == 8) {
        D.println("program ended");
        return;
      }
    }
    if (!f.exists()) {
      D.println("The initialisation file " + initFile + " was not found");
      switch (choice) {
        case 1:
          SQLConverter.requestInformation();
          break;
        case 2:
          XMLConverter.requestInformation();
          break;
        case 3:
          RDFSConverter.requestInformation();
          break;
        case 4:
          RDFSConverter.requestInformationN3();
          break;
        case 5:
          DBpediaLink.requestInformation();
          break;
        case 6:
          DBpediaExport.requestInformation();
          break;
        case 7:
          AlchConverter.requestInformation();
          break;
        default:
          break;
      }
    } else {
      D.println("The initialisation file " + initFile + " was found");
      switch (choice) {
        case 1:
          SQLConverter.fromIniFile(f);
          break;
        case 2:
          XMLConverter.fromIniFile(f);
          break;
        case 3:
          RDFSConverter.fromIniFile(f);
          break;
        case 4:
          RDFSConverter.fromIniFileN3(f);
          break;
        case 5:
          DBpediaLink.fromIniFile(f);
          break;
        case 6:
          DBpediaExport.fromIniFile(f);
          break;
        case 7:
          AlchConverter.fromIniFile(f);
          break;
        default:
          break;
      }
    }
    D.println("End of program");
  }
}
